// kraidmonitor.cpp
// Copyright (C) 2011, Gerald Pichler
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.




#include "kraidmonitor.h"
#include <QPainter>
#include <QFontMetrics>
#include <QSizeF>
#include <KLocale>
#include <kstandarddirs.h>
#include <QString>
#include <QDir>

#include <plasma/svg.h>
#include <plasma/theme.h>

kraidmonitor::kraidmonitor(QObject *parent, const QVariantList &args)
	: Plasma::Applet(parent, args),
	m_svg(this),
	g_icon(KStandardDirs::locate("icon", "default.kde4/scalable/apps/raidgreen.svg")),
	r_icon(KStandardDirs::locate("icon", "default.kde4/scalable/apps/raidred.svg")),
	y_icon(KStandardDirs::locate("icon", "default.kde4/scalable/apps/raidyellow.svg"))
{
	// this will get us the standard applet background, for free!
//	setBackgroundHints(DefaultBackground);
	QString greenIconName = KStandardDirs::locate("icon", "default.kde4/scalable/apps/raidgreen.svg");
	QString redIconName = KStandardDirs::locate("icon", "default.kde4/scalable/apps/raidred.svg");
	QString yellowIconName = KStandardDirs::locate("icon", "default.kde4/scalable/apps/raidyellow.svg");
/*	g_icon(greenIconName);
	r_icon(redIconName);
	y_icon(yellowIconName);
*/
//m_svg.setImagePath("widgets/background");
	setHasConfigurationInterface(true);
	resize(200, 200);
	
	goodArray = 0;
	
	//     QFile testFile(home + "/test.txt");
	//     testFile.open( IO_WriteOnly );
	//     QTextStream stream( &testFile );
	
	QString home = QDir::homePath();
	QString line;
	
	// if no ~/.kraidmonitor file exists, make one which contains the first available /dev/mdX device
	if (not(QFile::exists(home + QDir::separator() + ".kraidmonitor")))
	{
		QFile confFile(home + QDir::separator() + ".kraidmonitor");
		confFile.open(QIODevice::ReadWrite); QTextStream confStream( &confFile );
		
		QDir devMdX( "/dev", "md?" );
		devMdX.setFilter(QDir::System);
		line = "Device = " + devMdX.absolutePath() + devMdX.separator() + devMdX[0];
		confStream << line << "\n";
		
		confFile.close();
	}
	
	QFile confFile(home + QDir::separator() + ".kraidmonitor");
	confFile.open(QIODevice::ReadWrite);
	int i = 0;
	while( !confFile.atEnd() )
	{
		line = confFile.readLine();
		QString tmpStr1 = line.section('=', 1, 1);
		devices.append(tmpStr1.trimmed());
		i++;
	}
	confFile.close();
}


kraidmonitor::~kraidmonitor()
{
	delete[](threeSecond_timer);
}

void kraidmonitor::init()
{
	// kick off a refresh timer
	threeSecond_timer = new QTimer(this);
	connect(threeSecond_timer, SIGNAL(timeout()), this, SLOT(updateArrayStatus()));
	threeSecond_timer->start(3000);
}


void kraidmonitor::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *option, const QRect &contentsRect)
{
	p->setRenderHint(QPainter::SmoothPixmapTransform);
	p->setRenderHint(QPainter::Antialiasing);


	// We place the icon
	if (goodArray == 1) { p->drawPixmap(0, 0, g_icon.pixmap((int)contentsRect.width(),(int)contentsRect.height())); }
	else if (goodArray == 0) { p->drawPixmap(0, 0, y_icon.pixmap((int)contentsRect.width(),(int)contentsRect.height())); }
	else { p->drawPixmap(0, 0, r_icon.pixmap((int)contentsRect.width(),(int)contentsRect.height())); }
	
	p->save();
	p->restore();
}
void kraidmonitor::updateArrayStatus()
{
//	QFile testFile("/home/gerald/test.txt");
//	testFile.open( QIODevice::WriteOnly );
//	QTextStream stream( &testFile );

	QString line;
	QString device;
	for ( QStringList::Iterator dev = devices.begin(); dev != devices.end(); ++dev )
	{
		device = *dev;
		if (not(QFile::exists(device))) { goodArray = 0; }
		else
		{
			QString arrayName(device);
			arrayName.replace(QString("/dev/"), QString(""));

			//         /sys/block/md1/md/dev-loop0/state
			QString dirName(QString("/sys/block/") + arrayName + QString("/md/"));
			QDir mdDir(dirName);

			QStringList devFilter;
			devFilter << "dev-*";
			mdDir.setNameFilters(devFilter);
			QStringList deviceDirNames;
			deviceDirNames = mdDir.entryList();
			if (deviceDirNames.size() < 2) { goodArray = -1; }
			else
			{
				QFile mdStatusFile(dirName + QString("array_state"));
				mdStatusFile.open(QIODevice::ReadOnly);
				line = mdStatusFile.readLine();
				mdStatusFile.close();
				line = line.trimmed();
				if ((line == "clean") || (line == "active")) { goodArray = 1; }
				else { goodArray = -1; }

				for ( int i = 0; i < deviceDirNames.size(); ++i)
				{
					QString devDirName(deviceDirNames[i]);
					QFile devStatusFile(dirName + devDirName + QString("/state")); //stream << dirName << devDirName << QString("/state")<<"\n";
					devStatusFile.open(QIODevice::ReadOnly);
					line = devStatusFile.readLine();
					devStatusFile.close();
					line = line.trimmed(); //stream << line << "\n";
					if (line != "in_sync") { goodArray = -1; }
				}
			}
		}
		// force a re-draw
		update();
		if ( goodArray != 1){ break; }
	}


	//stream << device << "\n";
//	testFile.flush();
//	testFile.close();

}

#include "kraidmonitor.moc"
